@@ -1,18 +1,19 @@
 /*
- * Copyright (C) 2005 Claudio Ciccani <klan@users.sf.net>
+ * Copyright (C) 2005-2006 Claudio Ciccani <klan@users.sf.net>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
 
@@ -474,7 +475,7 @@ IFusionSoundMusicProvider_Mad_GetStreamDescription( IFusionSoundMusicProvider *t
      desc->flags      = FSSDF_SAMPLERATE | FSSDF_CHANNELS | FSSDF_BUFFERSIZE;
      desc->samplerate = data->samplerate;
      desc->channels   = data->channels;
-     desc->buffersize = data->samplerate >> 2;
+     desc->buffersize = data->samplerate/5;
 
      return DFB_OK;
 }
@@ -491,7 +492,7 @@ IFusionSoundMusicProvider_Mad_GetBufferDescription( IFusionSoundMusicProvider *t
      desc->flags      = FSBDF_SAMPLERATE | FSBDF_CHANNELS | FSBDF_LENGTH;
      desc->samplerate = data->samplerate;
      desc->channels   = data->channels;
-     desc->length     = data->samplerate >> 2;
+     desc->length     = data->samplerate/5;
 
      return DFB_OK;
 }
@@ -663,9 +664,13 @@ IFusionSoundMusicProvider_Mad_PlayToStream( IFusionSoundMusicProvider *thiz,
      data->dest.channels = desc.channels;
      data->dest.length   = desc.buffersize;
      
+     if (data->finished) {
+          direct_stream_seek( data->s, 0 );
+          data->finished = false;
+     }
+     
      /* start thread */
      data->playing  = true;
-     data->finished = false;
      data->thread   = direct_thread_create( DTT_DEFAULT, 
                                             MadStreamThread, data, "Mad" );
 
@@ -863,9 +868,13 @@ IFusionSoundMusicProvider_Mad_PlayToBuffer( IFusionSoundMusicProvider *thiz,
      data->callback = callback;
      data->ctx      = ctx;
      
+     if (data->finished) {
+          direct_stream_seek( data->s, 0 );
+          data->finished = false;
+     }
+   
      /* start thread */
      data->playing  = true;
-     data->finished = false;
      data->thread   = direct_thread_create( DTT_DEFAULT,
                                             MadBufferThread, data, "Mad" );
 
@@ -919,8 +928,9 @@ static DFBResult
 IFusionSoundMusicProvider_Mad_SeekTo( IFusionSoundMusicProvider *thiz,
                                       double                     seconds )
 {
-     DFBResult ret;
-     double    rate;
+     DFBResult    ret  = DFB_FAILURE;
+     double       rate;
+     unsigned int off;
      
      DIRECT_INTERFACE_GET_DATA( IFusionSoundMusicProvider_Mad )
 
@@ -931,12 +941,13 @@ IFusionSoundMusicProvider_Mad_SeekTo( IFusionSoundMusicProvider *thiz,
      
      rate = (data->desc.bitrate ? : data->frame.header.bitrate) >> 3;
      if (rate) {
-          ret = direct_stream_seek( data->s, seconds*rate );
-          if (ret == DFB_OK)
+          off = (seconds*rate);
+          ret = direct_stream_seek( data->s, off );
+          if (ret == DFB_OK) {
                data->seeked = true;
-     }
-     else {
-          ret = DFB_FAILURE;
+               if (direct_stream_length( data->s ))
+                    data->finished = (off == direct_stream_length( data->s ));
+          }
      }
      
      pthread_mutex_unlock( &data->lock );
@@ -995,9 +1006,17 @@ IFusionSoundMusicProvider_Mad_GetLength( IFusionSoundMusicProvider *thiz,
 static DFBResult
 Probe( IFusionSoundMusicProvider_ProbeContext *ctx )
 {
+     char *ext;
+     
      /* FIXME: detect by contents */
-     if (!strcasecmp( strrchr( ctx->filename, '.' ) ? : "", ".mp3" ))
-          return DFB_OK;
+     
+     ext = strrchr( ctx->filename, '.' );
+     if (ext) {
+          if (!strcasecmp( ext, ".mp1" ) ||
+              !strcasecmp( ext, ".mp2" ) ||
+              !strcasecmp( ext, ".mp3" ))
+               return DFB_OK;
+     }
           
      return DFB_UNSUPPORTED;
 }   
