@@ -1,18 +1,19 @@
 /*
- * Copyright (C) 2005 Claudio Ciccani <klan@users.sf.net>
+ * Copyright (C) 2005-2006 Claudio Ciccani <klan@users.sf.net>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  */
 
@@ -782,7 +783,10 @@ IFusionSoundMusicProvider_Wave_GetCapabilities( IFusionSoundMusicProvider   *thi
      if (!caps)
           return DFB_INVARG;
 
-     *caps = FMCAPS_BASIC | FMCAPS_SEEK;
+     if (direct_stream_seekable( data->stream ))
+          *caps = FMCAPS_BASIC | FMCAPS_SEEK;
+     else
+          *caps = FMCAPS_BASIC;
 
      return DFB_OK;
 }
@@ -871,7 +875,7 @@ IFusionSoundMusicProvider_Wave_GetStreamDescription( IFusionSoundMusicProvider *
                desc->sampleformat = FSSF_S32;
                break;
      }
-     desc->buffersize   = data->samplerate >> 2;
+     desc->buffersize   = data->samplerate/5;
 
      return DFB_OK;
 }
@@ -903,7 +907,7 @@ IFusionSoundMusicProvider_Wave_GetBufferDescription( IFusionSoundMusicProvider *
                desc->sampleformat = FSSF_S32;
                break;
      }
-     desc->length       = data->samplerate >> 2;
+     desc->length       = data->samplerate/5;
 
      return DFB_OK;
 }
@@ -1061,10 +1065,14 @@ IFusionSoundMusicProvider_Wave_PlayToStream( IFusionSoundMusicProvider *thiz,
      data->dest.format   = dst_format;
      data->dest.channels = desc.channels;
      data->dest.length   = desc.buffersize;
+
+     if (data->finished) {
+          direct_stream_seek( data->stream, data->headsize );
+          data->finished = false;
+     }
      
      /* start thread */
      data->playing  = true;
-     data->finished = false;
      data->thread   = direct_thread_create( DTT_DEFAULT,
                                             WaveStreamThread, data, "Wave" );
 
@@ -1241,12 +1249,16 @@ IFusionSoundMusicProvider_Wave_PlayToBuffer( IFusionSoundMusicProvider *thiz,
 
      data->callback      = callback;
      data->ctx           = ctx;
-     
+    
+     if (data->finished) {
+          direct_stream_seek( data->stream, data->headsize );
+          data->finished = false;
+     }
+    
      /* start thread */
-     data->playing  = true;
-     data->finished = false;
-     data->thread   = direct_thread_create( DTT_DEFAULT,
-                                            WaveBufferThread, data, "Wave" );
+     data->playing = true;
+     data->thread  = direct_thread_create( DTT_DEFAULT,
+                                           WaveBufferThread, data, "Wave" );
 
      pthread_mutex_unlock( &data->lock );
 
@@ -1308,12 +1320,16 @@ IFusionSoundMusicProvider_Wave_SeekTo( IFusionSoundMusicProvider *thiz,
 
      offset  = (double)data->samplerate * seconds;
      offset  = offset * data->channels * data->format >> 3;
-     if (data->datasize)
-          offset = MIN( offset, data->datasize );
+     if (data->datasize && offset > data->datasize)
+          return DFB_OK;
      offset += data->headsize;
 
      pthread_mutex_lock( &data->lock );
+     
      ret = direct_stream_seek( data->stream, offset );
+     if (ret == DFB_OK && data->datasize) 
+          data->finished = (offset == (data->headsize+data->datasize));
+     
      pthread_mutex_unlock( &data->lock );
      
      return ret;
