@@ -42,6 +42,7 @@
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
+#include <direct/signals.h>
 #include <direct/thread.h>
 #include <direct/util.h>
 
@@ -98,6 +99,8 @@ struct __FS_CoreSoundShared {
      FusionObjectPool    *buffer_pool;
      FusionObjectPool    *playback_pool;
 
+     FusionSHMPoolShared *shmpool;
+
      struct {
           DirectLink     *entries;
           FusionSkirmish  lock;
@@ -116,21 +119,30 @@ struct __FS_CoreSoundShared {
 };
 
 struct __FS_CoreSound {
-     int              refs;
+     int                  refs;
 
-     int              fusion_id;
+     int                  fusion_id;
 
-     FusionArena     *arena;
+     FusionWorld         *world;
+     FusionArena         *arena;
 
-     CoreSoundShared *shared;
+     CoreSoundShared     *shared;
 
-     bool             master;
-     int              fd;
-     DirectThread    *sound_thread;
+     bool                 master;
+     int                  fd;
+     DirectThread        *sound_thread;
+
+     DirectSignalHandler *signal_handler;
 };
 
 /******************************************************************************/
 
+static DirectSignalHandlerResult fs_core_signal_handler( int   num,
+                                                         void *addr,
+                                                         void *ctx );
+
+/******************************************************************************/
+
 static int fs_core_arena_initialize( FusionArena *arena,
                                      void        *ctx );
 static int fs_core_arena_join      ( FusionArena *arena,
@@ -145,16 +157,12 @@ static int fs_core_arena_shutdown  ( FusionArena *arena,
 /******************************************************************************/
 
 static CoreSound       *core_sound      = NULL;
-static pthread_mutex_t  core_sound_lock = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t  core_sound_lock = PTHREAD_MUTEX_INITIALIZER;
 
 DFBResult
 fs_core_create( CoreSound **ret_core )
 {
      int        ret;
-     int        world;
-#if FUSION_BUILD_MULTI
-     char       buf[16];
-#endif
      CoreSound *core;
 
      D_ASSERT( ret_core != NULL );
@@ -185,35 +193,32 @@ fs_core_create( CoreSound **ret_core )
           return DFB_NOSYSTEMMEMORY;
      }
 
-     core->fusion_id = fusion_init( dfb_config->session,
-#if DIRECT_BUILD_DEBUG
-                                    -DIRECTFB_CORE_ABI,
-#else
-                                    DIRECTFB_CORE_ABI,
-#endif
-                                    &world );
-     if (core->fusion_id < 0) {
+     ret = fusion_enter( fs_config->session, DIRECTFB_CORE_ABI, FER_ANY, &core->world );
+     if (ret) {
           D_FREE( core );
           pthread_mutex_unlock( &core_sound_lock );
-          return DFB_FUSION;
+          return ret;
      }
 
+     core->fusion_id = fusion_id( core->world );
+
 #if FUSION_BUILD_MULTI
-     D_DEBUG( "FusionSound/Core: world %d, fusion id %d\n", world, core->fusion_id );
-
-     snprintf( buf, sizeof(buf), "%d", world );
-
-     setenv( "DIRECTFB_SESSION", buf, true );
+     D_DEBUG( "FusionSound/Core: world %d, fusion id %d\n",
+              fusion_world_index( core->world ), core->fusion_id );
 #endif
 
      /* Initialize the references. */
      core->refs = 1;
 
+     direct_signal_handler_add( -1, fs_core_signal_handler, core, &core->signal_handler );
+
      /* Enter the FusionSound core arena. */
-     if (fusion_arena_enter( "FusionSound/Core",
+     if (fusion_arena_enter( core->world, "FusionSound/Core",
                              fs_core_arena_initialize, fs_core_arena_join,
                              core, &core->arena, &ret ) || ret)
      {
+          direct_signal_handler_remove( core->signal_handler );
+          fusion_exit( core->world, false );
           D_FREE( core );
           pthread_mutex_unlock( &core_sound_lock );
           return ret ? ret : DFB_FUSION;
@@ -229,7 +234,7 @@ fs_core_create( CoreSound **ret_core )
 }
 
 DFBResult
-fs_core_destroy( CoreSound *core )
+fs_core_destroy( CoreSound *core, bool emergency )
 {
      D_ASSERT( core != NULL );
      D_ASSERT( core == core_sound );
@@ -247,36 +252,33 @@ fs_core_destroy( CoreSound *core )
           return DFB_OK;
      }
 
+     direct_signal_handler_remove( core->signal_handler );
+     
      /* Exit the FusionSound core arena. */
      if (fusion_arena_exit( core->arena, fs_core_arena_shutdown,
                             core->master ? NULL : fs_core_arena_leave,
-                            core, false, NULL ) == DFB_BUSY)
+                            core, emergency, NULL ) == DFB_BUSY)
      {
-          D_WARN( "forking to wait until all slaves terminated" );
-
-          /* FIXME: quick hack to solve the dfb-slave-but-fs-master problem. */
-          switch (fork()) {
-               case -1:
-                    D_PERROR( "FusionSound/Core: fork() failed!\n" );
-
-                    while (fusion_arena_exit( core->arena, fs_core_arena_shutdown,
-                                              core->master ? NULL : fs_core_arena_leave,
-                                              core, false, NULL ) == DFB_BUSY)
-                         usleep( 100000 );
-
-                    break;
-
-               case 0:
-                    while (fusion_arena_exit( core->arena, fs_core_arena_shutdown,
-                                              core->master ? NULL : fs_core_arena_leave,
-                                              core, false, NULL ) == DFB_BUSY)
-                         usleep( 100000 );
-
-                    _exit(0);
+          if (core->master) {
+               if (emergency) {
+                    fusion_kill( core->world, 0, SIGKILL, 1000 );
+               }
+               else {
+                    fusion_kill( core->world, 0, SIGTERM, 5000 );
+                    fusion_kill( core->world, 0, SIGKILL, 2000 );
+               }
+          }
+          
+          while (fusion_arena_exit( core->arena, fs_core_arena_shutdown,
+                                    core->master ? NULL : fs_core_arena_leave,
+                                    core, emergency, NULL ) == DFB_BUSY)
+          {
+               D_ONCE( "waiting for FusionSound slaves to terminate" );
+               usleep( 100000 );
           }
      }
 
-     fusion_exit( false );
+     fusion_exit( core->world, emergency );
 
      /* Deallocate local core structure. */
      D_FREE( core );
@@ -298,7 +300,7 @@ fs_core_create_buffer( CoreSound *core )
      D_ASSERT( core->shared->buffer_pool != NULL );
 
      /* Create a new object in the buffer pool. */
-     return (CoreSoundBuffer*) fusion_object_create( core->shared->buffer_pool );
+     return (CoreSoundBuffer*) fusion_object_create( core->shared->buffer_pool, core->world );
 }
 
 CorePlayback *
@@ -309,7 +311,7 @@ fs_core_create_playback( CoreSound *core )
      D_ASSERT( core->shared->playback_pool != NULL );
 
      /* Create a new object in the playback pool. */
-     return (CorePlayback*) fusion_object_create( core->shared->playback_pool );
+     return (CorePlayback*) fusion_object_create( core->shared->playback_pool, core->world );
 }
 
 DirectResult
@@ -346,13 +348,13 @@ fs_core_add_playback( CoreSound    *core,
      shared = core->shared;
 
      /* Allocate playlist entry. */
-     entry = SHCALLOC( 1, sizeof(CorePlaylistEntry) );
+     entry = SHCALLOC( shared->shmpool, 1, sizeof(CorePlaylistEntry) );
      if (!entry)
           return DFB_NOSYSTEMMEMORY;
 
      /* Link playback to playlist entry. */
      if (fs_playback_link( &entry->playback, playback )) {
-          SHFREE( entry );
+          SHFREE( shared->shmpool, entry );
           return DFB_FUSION;
      }
 
@@ -387,7 +389,7 @@ fs_core_remove_playback( CoreSound    *core,
 
                fs_playback_unlink( &entry->playback );
 
-               SHFREE( entry );
+               SHFREE( shared->shmpool, entry );
           }
      }
 
@@ -404,6 +406,38 @@ fs_core_output_delay( CoreSound *core )
      return core->shared->output_delay;
 }
 
+FusionWorld *
+fs_core_world( CoreSound *core )
+{
+     D_ASSERT( core != NULL );
+
+     return core->world;
+}
+
+FusionSHMPoolShared *
+fs_core_shmpool( CoreSound *core )
+{
+     D_ASSERT( core != NULL );
+     D_ASSERT( core->shared != NULL );
+
+     return core->shared->shmpool;
+}
+
+/******************************************************************************/
+
+static DirectSignalHandlerResult
+fs_core_signal_handler( int num, void *addr, void *ctx )
+{
+     CoreSound *core = (CoreSound*) ctx;
+
+     D_ASSERT( core != NULL );
+     D_ASSERT( core == core_sound );
+
+     fs_core_destroy( core, true );
+
+     return DFB_OK;
+}
+
 /******************************************************************************/
 
 static void *
@@ -473,7 +507,7 @@ sound_thread( DirectThread *thread, void *arg )
 
                     fs_playback_unlink( &entry->playback );
 
-                    SHFREE( entry );
+                    SHFREE( shared->shmpool, entry );
                }
           }
 
@@ -485,7 +519,7 @@ sound_thread( DirectThread *thread, void *arg )
                     if (shared->config.channels == 1) {
                          for (i = 0; i < samples; i += 2) {
                               register __fsf s;
-                              s = fsf_round_u8( fsf_add( mixing[i], mixing[i+1] ) );
+                              s = fsf_add( mixing[i], mixing[i+1] );
                               s = fsf_shr( s, 1 );
                               s = fsf_clip( s );                  
                               output[i>>1] = fsf_to_u8( s );
@@ -493,7 +527,7 @@ sound_thread( DirectThread *thread, void *arg )
                     } else {      
                          for (i = 0; i < samples; i++) {
                               register __fsf s;
-                              s = fsf_round_u8( mixing[i] );                       
+                              s = mixing[i];                       
                               s = fsf_clip( s );                  
                               output[i] = fsf_to_u8( s );
                          }
@@ -503,7 +537,7 @@ sound_thread( DirectThread *thread, void *arg )
                     if (shared->config.channels == 1) {
                          for (i = 0; i < samples; i += 2) {
                               register __fsf s;
-                              s = fsf_round_s16( fsf_add( mixing[i], mixing[i+1] ) );
+                              s = fsf_add( mixing[i], mixing[i+1] );
                               s = fsf_shr( s, 1 );
                               s = fsf_clip( s );                         
                               ((__s16*)output)[i>>1] = fsf_to_s16( s );
@@ -511,7 +545,7 @@ sound_thread( DirectThread *thread, void *arg )
                     } else {
                          for (i = 0; i < samples; i++) {
                               register __fsf s;
-                              s = fsf_round_s16( mixing[i] );
+                              s = mixing[i];
                               s = fsf_clip( s );                         
                               ((__s16*)output)[i] = fsf_to_s16( s );
                          }
@@ -522,7 +556,7 @@ sound_thread( DirectThread *thread, void *arg )
                          for (i = 0; i < samples; i += 2) {
                               register __fsf s;
                               register int   d;
-                              s = fsf_round_s24( fsf_add( mixing[i], mixing[i+1] ) );
+                              s = fsf_add( mixing[i], mixing[i+1] );
                               s = fsf_shr( s, 1 );
                               s = fsf_clip( s );
                               d = fsf_to_s24( s );
@@ -540,7 +574,7 @@ sound_thread( DirectThread *thread, void *arg )
                          for (i = 0; i < samples; i++) {
                               register __fsf s;
                               register int   d;
-                              s = fsf_round_s24( mixing[i] );
+                              s = mixing[i];
                               s = fsf_clip( s );
                               d = fsf_to_s24( s );
 #ifdef WORDS_BIGENDIAN
@@ -559,7 +593,7 @@ sound_thread( DirectThread *thread, void *arg )
                     if (shared->config.channels == 1) {
                          for (i = 0; i < samples; i += 2) {
                               register __fsf s;
-                              s = fsf_round_s32( fsf_add( mixing[i], mixing[i+1] ) );
+                              s = fsf_add( mixing[i], mixing[i+1] );
                               s = fsf_shr( s, 1 );
                               s = fsf_clip( s );                         
                               ((__s32*)output)[i>>1] = fsf_to_s32( s );
@@ -567,7 +601,7 @@ sound_thread( DirectThread *thread, void *arg )
                     } else {
                          for (i = 0; i < samples; i++) {
                               register __fsf s;
-                              s = fsf_round_s32( mixing[i] );
+                              s = mixing[i];
                               s = fsf_clip( s );                         
                               ((__s32*)output)[i] = fsf_to_s32( s );
                          }
@@ -598,11 +632,11 @@ fs_core_initialize( CoreSound *core )
      int              channels = shared->config.channels;
      int              rate     = shared->config.rate;
 
-     /* open sound device */
+     /* open sound device in non-blocking mode */
      if (fs_config->device)
-          fd = open( fs_config->device, O_WRONLY );
+          fd = open( fs_config->device, O_WRONLY | O_NONBLOCK );
      else
-          fd = direct_try_open( "/dev/dsp", "/dev/sound/dsp", O_WRONLY, true );
+          fd = direct_try_open( "/dev/dsp", "/dev/sound/dsp", O_WRONLY | O_NONBLOCK, true );
      
      if (fd < 0) {
           D_ERROR( "FusionSound/Core: "
@@ -610,13 +644,13 @@ fs_core_initialize( CoreSound *core )
           return DFB_INIT;
      }
 
+     /* reset to blocking mode */
+     fcntl( fd, F_SETFL, fcntl( fd, F_GETFL ) & ~O_NONBLOCK );
+
      /* set application profile */
 #ifdef SNDCTL_DSP_PROFILE
-     if (ioctl( fd, SNDCTL_DSP_PROFILE, &prof ) == -1) {
-          D_ERROR( "FusionSound/Core: Unable to set application profile!\n" );
-          close( fd );
-          return DFB_UNSUPPORTED;
-     }
+     if (ioctl( fd, SNDCTL_DSP_PROFILE, &prof ) == -1)
+          D_WARN( "FusionSound/Core: Unable to set application profile!\n" );
 #endif
      
      /* set bits per sample */
@@ -700,13 +734,13 @@ fs_core_initialize( CoreSound *core )
      core->fd = fd;
 
      /* initialize playback list lock */
-     fusion_skirmish_init( &shared->playlist.lock, "FusionSound playlist list lock" );
+     fusion_skirmish_init( &shared->playlist.lock, "FusionSound Playlist", core->world );
 
      /* create a pool for sound buffer objects */
-     shared->buffer_pool = fs_buffer_pool_create();
+     shared->buffer_pool = fs_buffer_pool_create( core->world );
 
      /* create a pool for playback objects */
-     shared->playback_pool = fs_playback_pool_create();
+     shared->playback_pool = fs_playback_pool_create( core->world );
 
      /* create sound thread */
      core->sound_thread = direct_thread_create( DTT_CRITICAL, sound_thread, core, "Sound Mixer" );
@@ -763,14 +797,14 @@ fs_core_shutdown( CoreSound *core )
 
           fs_playback_unlink( &entry->playback );
 
-          SHFREE( entry );
+          SHFREE( shared->shmpool, entry );
      }
 
      /* destroy playback object pool */
-     fusion_object_pool_destroy( shared->playback_pool );
+     fusion_object_pool_destroy( shared->playback_pool, core->world );
 
      /* destroy buffer object pool */
-     fusion_object_pool_destroy( shared->buffer_pool );
+     fusion_object_pool_destroy( shared->buffer_pool, core->world );
 
      /* destroy playlist lock */
      fusion_skirmish_destroy( &shared->playlist.lock );
@@ -784,27 +818,31 @@ static int
 fs_core_arena_initialize( FusionArena *arena,
                           void        *ctx )
 {
-     DFBResult        ret;
-     CoreSound       *core   = ctx;
-     CoreSoundShared *shared;
+     DFBResult            ret;
+     CoreSound           *core   = ctx;
+     CoreSoundShared     *shared;
+     FusionSHMPoolShared *pool;
 
      D_DEBUG( "FusionSound/Core: Initializing...\n" );
 
-/*     if (!dfb_core_is_master()) {
-          ERRORMSG( "FusionSound/Core: Only master can initialize for now!\n" );
-          return DFB_INIT;
-     }*/
+     /* Create the shared memory pool first! */
+     ret = fusion_shm_pool_create( core->world, "FusionSound Main Pool", 0x1000000,
+                                   direct_config->debug, &pool );
+     if (ret)
+          return ret;
 
-     /* Allocate shared structure. */
-     shared = SHCALLOC( 1, sizeof(CoreSoundShared) );
+     /* Allocate shared structure in the new pool. */
+     shared = SHCALLOC( pool, 1, sizeof(CoreSoundShared) );
      if (!shared) {
-          D_ERROR( "FusionSound/Core: Could not allocate (shared) memory!\n" );
-          return DFB_NOSYSTEMMEMORY;
+          fusion_shm_pool_destroy( core->world, pool );
+          return D_OOSHM();
      }
 
      core->shared = shared;
      core->master = true;
 
+     shared->shmpool = pool;
+
      /* FIXME: add live configuration */
      switch (fs_config->sampleformat) {
           case FSSF_U8:
@@ -830,7 +868,8 @@ fs_core_arena_initialize( FusionArena *arena,
      /* Initialize. */
      ret = fs_core_initialize( core );
      if (ret) {
-          SHFREE( shared );
+          SHFREE( pool, shared );
+          fusion_shm_pool_destroy( core->world, pool );
           return ret;
      }
 
@@ -840,6 +879,39 @@ fs_core_arena_initialize( FusionArena *arena,
      return DFB_OK;
 }
 
+static int
+fs_core_arena_shutdown( FusionArena *arena,
+                        void        *ctx,
+                        bool         emergency)
+{
+     DFBResult            ret;
+     CoreSound           *core = ctx;
+     CoreSoundShared     *shared;
+     FusionSHMPoolShared *pool;
+
+     shared = core->shared;
+
+     pool = shared->shmpool;
+
+     D_DEBUG( "FusionSound/Core: Shutting down...\n" );
+
+     if (!core->master) {
+          D_DEBUG( "FusionSound/Core: Refusing shutdown in slave.\n" );
+          return fs_core_leave( core );
+     }
+
+     /* Shutdown. */
+     ret = fs_core_shutdown( core );
+     if (ret)
+          return ret;
+
+     SHFREE( pool, shared );
+
+     fusion_shm_pool_destroy( core->world, pool );
+
+     return DFB_OK;
+}
+
 static int
 fs_core_arena_join( FusionArena *arena,
                     void        *ctx )
@@ -882,26 +954,3 @@ fs_core_arena_leave( FusionArena *arena,
      return DFB_OK;
 }
 
-static int
-fs_core_arena_shutdown( FusionArena *arena,
-                        void        *ctx,
-                        bool         emergency)
-{
-     DFBResult  ret;
-     CoreSound *core = ctx;
-
-     D_DEBUG( "FusionSound/Core: Shutting down...\n" );
-
-     if (!core->master) {
-          D_DEBUG( "FusionSound/Core: Refusing shutdown in slave.\n" );
-          return fs_core_leave( core );
-     }
-
-     /* Shutdown. */
-     ret = fs_core_shutdown( core );
-     if (ret)
-          return ret;
-
-     return DFB_OK;
-}
-
